
It is common in the scientific literature to encounter uncertain data values
which are reported as following a specific distribution. For example, an author
report the mean and standard deviation of a value stated to follow a
normal distribution. `UncertainData` makes it easy to represent such values!


## Supported distributions

Supported distributions are `Uniform`, `Normal`, `Gamma`, `Beta`, `BetaPrime`,
`Frechet`, `Binomial`, `BetaBinomial` (more distributions will be added in the
future!).



## Constructors

There are two constructors that creates uncertain values
represented by theoretical distributions. Parameters are provided
to the constructor in the same order as for constructing the equivalent
distributions in `Distributions.jl`.

Uncertain values represented by theoretical distributions may be constructed
using the two-parameter or three-parameter constructors
`UncertainValue(d::Type{D}, a<:Number, b<:Number)` or
`UncertainValue(d::Type{D}, a<:Number, b<:Number, c<:Number)` (see below).


### Two-parameter distributions

```@docs
UncertainValue(distribution::Type{D}, a::T1, b::T2; kwargs...) where {T1<:Number, T2 <: Number, D<:Distribution}
```


### Three-parameter distributions

```@docs
UncertainValue(distribution::Type{D}, a::T1, b::T2, c::T3; kwargs...) where {T1<:Number, T2<:Number, T3<:Number, D<:Distribution}
```


## Examples

``` julia tab="Uniform"
# Uncertain value generated by a uniform distribution on [-5.0, 5.1].
uv = UncertainValue(Uniform, -5.0, 5.1)
```

``` julia tab="Normal"
# Uncertain value generated by a normal distribution with parameters μ = -2 and
# σ = 0.5.
uv = UncertainValue(Normal, -2, 0.5)
```

``` julia tab="Gamma"
# Uncertain value generated by a gamma distribution with parameters α = 2.2
# and θ = 3.
uv = UncertainValue(Gamma, 2.2, 3)
```

``` julia tab="Beta"
# Uncertain value generated by a beta distribution with parameters α = 1.5
# and β = 3.5
uv = UncertainValue(Beta, 1.5, 3.5)
```

``` julia tab="BetaPrime"
# Uncertain value generated by a beta prime distribution with parameters α = 1.7
# and β = 3.2
uv = UncertainValue(Beta, 1.7, 3.2)
```

``` julia tab="Fréchet"
# Uncertain value generated by a Fréchet distribution with parameters α = 2.1
# and θ = 4
uv = UncertainValue(Beta, 2.1, 4)
```

``` julia tab="Binomial"
# Uncertain value generated by binomial distribution with n = 28 trials and
# probability p = 0.2 of success in individual trials.
uv = UncertainValue(Binomial, 28, 0.2)
```

``` julia tab="BetaBinomial"
# Creates an uncertain value generated by a beta-binomial distribution with
# n = 28 trials, and parameters α = 1.5 and β = 3.5.
uv = UncertainValue(BetaBinomial, 28, 3.3, 4.4)
```

